@@ -1,5 +1,5 @@
 /*
-    $Id: cdio_private.h,v 1.29 2005/03/23 11:26:49 rocky Exp $
+    $Id: cdio_private.h,v 1.31 2006/01/23 20:48:11 rocky Exp $
 
     Copyright (C) 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com>
 
@@ -24,7 +24,7 @@
 #ifndef __CDIO_PRIVATE_H__
 #define __CDIO_PRIVATE_H__
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H) && !defined(LIBCDIO_CONFIG_H)
 # include "config.h"
 #endif
 
