@@ -1,7 +1,7 @@
 /*
-    $Id: udf_fs.c,v 1.15 2006/03/18 00:53:20 rocky Exp $
+    $Id: udf_fs.c,v 1.21 2006/04/28 16:38:23 rocky Exp $
 
-    Copyright (C) 2005 Rocky Bernstein <rocky@panix.com>
+    Copyright (C) 2005, 2006 Rocky Bernstein <rocky@cpan.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -69,6 +69,7 @@ const char VSD_STD_ID_TEA01[] = {'T', 'E', 'A', '0', '1'};
 
 #include <cdio/bytesex.h>
 #include "udf_private.h"
+#include "udf_fs.h"
 
 /*
  * The UDF specs are pretty clear on how each data structure is made
@@ -121,8 +122,8 @@ udf_new_dirent(udf_file_entry_t *p_udf_fe, udf_t *p_udf,
  * Check the descriptor tag for both the correct id and correct checksum.
  * Return zero if all is good, -1 if not.
  */
-static int
-udf_checktag(udf_tag_t *p_tag, udf_Uint16_t tag_id)
+int
+udf_checktag(const udf_tag_t *p_tag, udf_Uint16_t tag_id)
 {
   uint8_t *itag;
   uint8_t i;
@@ -153,6 +154,7 @@ udf_get_lba(const udf_file_entry_t *p_udf_fe,
   switch (p_udf_fe->icb_tag.flags & ICBTAG_FLAG_AD_MASK) {
   case ICBTAG_FLAG_AD_SHORT:
     {
+      /* The allocation descriptor field is filled with short_ad's. */
       udf_short_ad_t *p_ad = (udf_short_ad_t *) 
 	(p_udf_fe->ext_attr + p_udf_fe->i_extended_attr);
       
@@ -164,6 +166,7 @@ udf_get_lba(const udf_file_entry_t *p_udf_fe,
     break;
   case ICBTAG_FLAG_AD_LONG:
     {
+      /* The allocation descriptor field is filled with long_ad's */
       udf_long_ad_t *p_ad = (udf_long_ad_t *) 
 	(p_udf_fe->ext_attr + p_udf_fe->i_extended_attr);
       
@@ -192,6 +195,11 @@ udf_get_lba(const udf_file_entry_t *p_udf_fe,
 
 #define udf_PATH_DELIMITERS "/\\"
 
+/* Searches p_udf_dirent a directory entry called psz_token.
+   Note p_udf_dirent is continuously updated. If the entry is
+   not found p_udf_dirent is useless and thus the caller should
+   not use it afterwards.
+*/
 static 
 udf_dirent_t *
 udf_ff_traverse(udf_dirent_t *p_udf_dirent, char *psz_token)
@@ -208,13 +216,14 @@ udf_ff_traverse(udf_dirent_t *p_udf_dirent, char *psz_token)
 	if (p_udf_dirent2) {
 	  udf_dirent_t * p_udf_dirent3 = 
 	    udf_ff_traverse(p_udf_dirent2, next_tok);
-	  
-	  if (!p_udf_dirent3) udf_dirent_free(p_udf_dirent2);
+
+	  /* if p_udf_dirent3 is null p_udf_dirent2 is free'd. */
 	  return p_udf_dirent3;
 	}
       }
     }
   }
+  free(p_udf_dirent->psz_name);
   return NULL;
 }
 
@@ -232,8 +241,18 @@ udf_fopen(udf_dirent_t *p_udf_root, const char *psz_name)
     
     strncpy(tokenline, psz_name, udf_MAX_PATHLEN);
     psz_token = strtok(tokenline, udf_PATH_DELIMITERS);
-    if (psz_token)
-      p_udf_file = udf_ff_traverse(p_udf_root, psz_token);
+    if (psz_token) {
+      /*** FIXME??? udf_dirent can be variable size due to the
+	   extended attributes and descriptors. Given that, is this
+	   correct? 
+       */
+      udf_dirent_t *p_udf_dirent = 
+	udf_new_dirent(&p_udf_root->fe, p_udf_root->p_udf,
+		       p_udf_root->psz_name, p_udf_root->b_dir, 
+		       p_udf_root->b_parent);
+      p_udf_file = udf_ff_traverse(p_udf_dirent, psz_token);
+      udf_dirent_free(p_udf_dirent);
+    }
     else if ( 0 == strncmp("/", psz_name, sizeof("/")) ) {
       return udf_new_dirent(&p_udf_root->fe, p_udf_root->p_udf,
 			    p_udf_root->psz_name, p_udf_root->b_dir, 
@@ -267,9 +286,13 @@ static udf_dirent_t *
 udf_new_dirent(udf_file_entry_t *p_udf_fe, udf_t *p_udf,
 	       const char *psz_name, bool b_dir, bool b_parent) 
 {
-  udf_dirent_t *p_udf_dirent = 
-    (udf_dirent_t *) calloc(1, sizeof(udf_dirent_t));
+  const unsigned int i_alloc_size = p_udf_fe->i_alloc_descs
+    + p_udf_fe->i_extended_attr;
+  
+  udf_dirent_t *p_udf_dirent = (udf_dirent_t *) 
+    calloc(1, sizeof(udf_dirent_t) + i_alloc_size);
   if (!p_udf_dirent) return NULL;
+  
   p_udf_dirent->psz_name     = strdup(psz_name);
   p_udf_dirent->b_dir        = b_dir;
   p_udf_dirent->b_parent     = b_parent;
@@ -277,9 +300,10 @@ udf_new_dirent(udf_file_entry_t *p_udf_fe, udf_t *p_udf,
   p_udf_dirent->i_part_start = p_udf->i_part_start;
   p_udf_dirent->dir_left     = uint64_from_le(p_udf_fe->info_len); 
 
-  memcpy(&(p_udf_dirent->fe), p_udf_fe, sizeof(udf_file_entry_t));
-  udf_get_lba( p_udf_fe, &(p_udf_dirent->dir_lba), 
-	       &(p_udf_dirent->dir_end_lba) );
+  memcpy(&(p_udf_dirent->fe), p_udf_fe, 
+	 sizeof(udf_file_entry_t) + i_alloc_size);
+  udf_get_lba( p_udf_fe, &(p_udf_dirent->i_loc), 
+	       &(p_udf_dirent->i_loc_end) );
   return p_udf_dirent;
 }
 
@@ -603,14 +627,14 @@ udf_readdir(udf_dirent_t *p_udf_dirent)
   
   if (!p_udf_dirent->fid) {
     uint32_t i_sectors = 
-      (p_udf_dirent->dir_end_lba - p_udf_dirent->dir_lba + 1);
+      (p_udf_dirent->i_loc_end - p_udf_dirent->i_loc + 1);
     uint32_t size = UDF_BLOCKSIZE * i_sectors;
     driver_return_code_t i_ret;
 
     if (!p_udf_dirent->sector)
       p_udf_dirent->sector = (uint8_t*) malloc(size);
     i_ret = udf_read_sectors(p_udf, p_udf_dirent->sector, 
-			     p_udf_dirent->i_part_start+p_udf_dirent->dir_lba, 
+			     p_udf_dirent->i_part_start+p_udf_dirent->i_loc, 
 			     i_sectors);
     if (DRIVER_OP_SUCCESS == i_ret)
       p_udf_dirent->fid = (udf_fileid_desc_t *) p_udf_dirent->sector;
@@ -638,7 +662,9 @@ udf_readdir(udf_dirent_t *p_udf_dirent)
 	udf_read_sectors(p_udf, p_udf_fe, p_udf->i_part_start 
 			 + p_udf_dirent->fid->icb.loc.lba, 1);
       
-	memcpy(&(p_udf_dirent->fe), p_udf_fe, sizeof(udf_file_entry_t));
+	memcpy(&(p_udf_dirent->fe), p_udf_fe, 
+	       sizeof(udf_file_entry_t) + p_udf_fe->i_alloc_descs 
+	       + p_udf_fe->i_extended_attr );
 
 	if (strlen(p_udf_dirent->psz_name) < i_len) 
 	  p_udf_dirent->psz_name = (char *)
